import {
  Form,
  Row,
  Input,
  Select,
  Col,
  Typography,
  Button,
  Menu,
  Dropdown,
  message,
  Modal,
} from 'antd';
import { useDispatch, useIntl, useSelector } from 'umi';
import styles from '../../styles.module.less';
import {
  CallBackLinks,
  IDataCenter,
  EditProductVariant,
  IDataCenterState,
  IEditProduct,
  IEditProductState, EditProductPricingTier,
} from '@/typings';
import Buttons from '@/pages/Product/ProductEdit/Steps/Buttons';
import { errorHandler } from '@/utils/errorHandler';
import { editProductSave, saveStateAction } from '@/pages/Product/ProductEdit/models';
import { useSafeState, useUpdateEffect } from 'ahooks';
import React from 'react';
import { history } from '@@/core/history';
import { setLinkCallBack } from '@/models/linkCallbacks';
import { IconMoon } from '@/components/IconMoon/IconMoon';

const positiveIntegerValidate = (_: any, value: any, intl: any, required: boolean) => {
  if (!required && !value) return Promise.resolve();
  else if (!value) return Promise.reject();

  const pattern = /^[+]?\d+([.]\d+)?$/;
  if (!pattern.test(value.toString()) || value <= 0)
    return Promise.reject(
      new Error(intl.formatMessage({ id: 'product.positiveWholeNumberOnly' })),
    );
  return Promise.resolve();
};

const dataToFormValues = (variants: EditProductVariant[] | null) => {
  const initValues: { [name: string]: any } = {};
  if (!variants) return {};
  variants.forEach((e, index) => {
    initValues[`${index}_variantAttribute`] = e.variantAttribute;
    initValues[`${index}_variantName`] = e.variantName;
    initValues[`${index}_simpleSku`] = e.simpleSku;
    initValues[`${index}_weight`] = e.weight;
    initValues[`${index}_height`] = e.height;
    initValues[`${index}_width`] = e.width;
    initValues[`${index}_length`] = e.length;
  });
  return initValues;
};

const formValuesToData = (
  formValues: Dictionary<any>,
  variants: EditProductVariant[],
) => {
  const updatedVariants = variants.map((e) => ({ ...e }));
  Object.entries(formValues).map(([key, value]) => {
    const [index, attribute] = key.split('_');
    const intIndex = parseInt(index);
    const updatedVariant = updatedVariants[intIndex];
    updatedVariant[attribute] = value;
  });
  return updatedVariants;
};

const NewDefaultTierPricing: EditProductPricingTier[] = [
  {
    qty: [1],
    currency: 'USD',
    percentage: 0,
    price: 0,
  }
]

export default function () {
  const intl = useIntl();
  const [form] = Form.useForm();
  const dispatch = useDispatch();

  const editProduct = useSelector<IEditProductState, IEditProduct>((state) => {
    return state.editProduct;
  });

  const dataCenter = useSelector<IDataCenterState, IDataCenter>(
    (state) => state.dataCenter,
  );
  const [selectedVariant, setSelectedVariant] = useSafeState<number>(0);
  const [disable, setDisable] = useSafeState({ save: true, nextStep: true });

  const checkIfAllFillIn = (values: any, allValues: any, firstTimeCheck = false) => {
    if (!editProduct.data.variants?.length)
      return setDisable({ nextStep: true, save: true });
    const requiredFields = ['variantAttribute', 'simpleSku', 'weight'];
    const requiredFieldsOnly = Object.entries(allValues).filter(([key, value]) => {
      const isRequiredField = requiredFields.find((e) => key.includes(e));
      if (!isRequiredField) return false;
      if (!value) return false;
      return true;
    });
    const numRequiredFields = editProduct.data.variants.reduce(
      (accumulator, e) =>
        e.variantAttribute === 'package' ? accumulator + 3 : accumulator + 1,
      0,
    );
    if (firstTimeCheck && requiredFieldsOnly.length === numRequiredFields)
      return setDisable({ save: true, nextStep: false });
    if (requiredFieldsOnly.length < numRequiredFields)
      return setDisable({ save: true, nextStep: true });
    setDisable({ nextStep: false, save: false });
  };

  const validateFields = async () => {
    try {
      await form.validateFields();
      return { valid: true };
    } catch (error) {
      return { valid: false, error };
    }
  };

  const onSubmit = async (advanceStep = false) => {
    try {
      const res = await validateFields();
      if (!res.valid) {
        const errorField = (res.error as any).errorFields[0];
        const name = errorField.name[0];
        const [index] = name.split('_');
        const intIndex = parseInt(index);
        setSelectedVariant(intIndex);
        // one more validate to highlight the fields
        await validateFields();
        return;
      }
      const values = form.getFieldsValue();
      const updatedVariants = formValuesToData(values, editProduct.data.variants || []);
      return saveStateAction(
        dispatch,
        editProduct,
        { variants: updatedVariants },
        editProduct.current + (advanceStep ? 1 : 0),
        !disable.save,
      );
    } catch (error) {
      errorHandler(error);
    }
  };

  const onRedirectLink = async () => {
    const valid = await validateFields();
    const promise = new Promise((resolve) => {
      if (disable.save && !disable.nextStep) return resolve(true);
      if (!valid)
        Modal.confirm({
          title: intl.formatMessage({
            id: 'product.notCompletedPopup',
          }),
          onOk: () => resolve(true),
          onCancel: () => resolve(false),
          cancelText: intl.formatMessage({ id: 'product.cancel' }),
          okText: intl.formatMessage({ id: 'product.quit' }),
        });
      if (valid)
        Modal.confirm({
          title: intl.formatMessage({ id: 'product.wouldYouLikeToSaveChanges?' }),
          onOk: async () => {
            const success = await onSubmit();
            const isNewProduct = editProduct.isNew;
            const messageText = isNewProduct
              ? intl.formatMessage({ id: 'product.theProductDraftHasBeenSaved' })
              : intl.formatMessage({ id: 'product.productHasBeenSaved' });
            if (success) message.success(messageText);
            resolve(success);
          },
          onCancel: () => resolve(true),
          cancelText: intl.formatMessage({ id: 'product.discard' }),
          okText: intl.formatMessage({ id: 'product.save' }),
        });
    });
    return promise;
  };

  const resetGoBackCallBack = () => {
    const regex = /^\/home\/product\/edit-product\//;
    const isEdit = regex.test(history.location.pathname);
    const callBackLink: CallBackLinks = isEdit
      ? '/home/product/edit-product/:id'
      : '/home/product/create-product';
    dispatch({
      type: setLinkCallBack,
      payload: {
        [callBackLink]: onRedirectLink,
      },
    });
  };

  React.useEffect(() => {
    resetGoBackCallBack();
  }, [disable]);

  React.useEffect(() => {
    checkIfAllFillIn(null, dataToFormValues(editProduct.data.variants) || [], true);
  }, []);

  useUpdateEffect(() => {
    form.resetFields();
    const initValues = dataToFormValues(editProduct.data.variants);
    form.resetFields();
    form.setFieldsValue(initValues);
    checkIfAllFillIn(null, initValues);
  }, [editProduct.data.variants]);

  const onNext = async () => {
    if (!editProduct.data.variants?.length) {
      message.warning(intl.formatMessage({ id: 'product.noVariantError' }));
      return false;
    }
    const result = (await onSubmit(true)) as boolean;
    if (!result) setDisable({ nextStep: true, save: true });
    return result;
  };

  const getDownButton = (index: number) => {
    const dropDownMenu = (
      <Menu
        items={[
          {
            key: '1',
            label: (
              <a
                onClick={(e) => {
                  e.stopPropagation();
                  const newVariants = [...editProduct.data.variants!];
                  const values = form.getFieldsValue();
                  const updatedVariants = formValuesToData(values, newVariants);
                  updatedVariants.splice(index, 1);
                  dispatch({
                    type: editProductSave,
                    payload: { data: { ...editProduct.data, variants: updatedVariants } },
                  });
                  setSelectedVariant(0);
                }}
              >
                {intl.formatMessage({ id: 'product.delete' })}
              </a>
            ),
          },
        ]}
      />
    );
    return (
      <Dropdown
        getPopupContainer={(e) => e.parentElement || e}
        overlay={dropDownMenu}
        placement={'bottomRight'}
        arrow={true}
      >
        <Button
          className={styles.variantDeleteBtn}
          type={'link'}
          icon={<IconMoon size={30} icon={'Ellipsis'} />}
        />
      </Dropdown>
    );
  };

  const onAddVariant = () => {
    const values = form.getFieldsValue();
    const newVariant: EditProductVariant = {
      widthUom: dataCenter!.measureUnits.length[0].code,
      lengthUom: dataCenter!.measureUnits.length[0].code,
      heightUom: dataCenter!.measureUnits.length[0].code,
      weightUom: dataCenter!.measureUnits.weight[0].code,
      unitPriceCurrency: 'USD',
      tierPricing: [...NewDefaultTierPricing],
    };
    const newVariantsList = editProduct.data.variants
      ? [...editProduct.data.variants, newVariant]
      : [newVariant];
    const updatedVariants = formValuesToData(values, newVariantsList);
    dispatch({
      type: editProductSave,
      payload: { data: { ...editProduct.data, variants: updatedVariants } },
    });
  };

  const onNameChange = (index: number, value: string) => {
    const newVariantList = editProduct.data.variants!.map((e) => ({ ...e }));
    newVariantList[index].variantName = value;
    const values = form.getFieldsValue();
    const updatedVariants = formValuesToData(values, newVariantList);
    dispatch({
      type: editProductSave,
      payload: { data: { ...editProduct.data, variants: updatedVariants } },
    });
  };

  const onVariantsSelectChange = (index: number, value: string) => {
    const newVariantList = editProduct.data.variants!.map((e) => ({ ...e }));
    newVariantList[index].variantAttribute = value;
    const values = form.getFieldsValue();
    const updatedVariants = formValuesToData(values, newVariantList);
    dispatch({
      type: editProductSave,
      payload: { data: { ...editProduct.data, variants: updatedVariants } },
    });
  };

  const onUomChange = (index: number, value: string, field: string) => {
    const newVariantList = editProduct.data.variants!.map((e) => ({ ...e }));
    newVariantList[index][field] = value;
    const values = form.getFieldsValue();
    const updatedVariants = formValuesToData(values, newVariantList);
    dispatch({
      type: editProductSave,
      payload: { data: { ...editProduct.data, variants: updatedVariants } },
    });
  };

  return (
    <Row
      gutter={[32, 16]}
      className={[styles.stepContainer, styles.variantsStepContainer].join(' ')}
    >
      <Col className={styles.leftHalfCol}>
        <Typography.Title level={4}>
          {intl.formatMessage({ id: 'product.variants' })}
        </Typography.Title>
        {editProduct.data.variants?.map((e, index) => {
          return (
            <Row
              key={`${e.simpleSku}_${index}`}
              onClick={() => setSelectedVariant(index)}
              justify={'space-between'}
              className={[
                styles.variantRow,
                selectedVariant === index ? styles.selectedVariantRow : '',
              ].join(' ')}
            >
              <Col>
                {e.variantName || intl.formatMessage({ id: 'product.newVariant' })}
              </Col>
              <Col>{getDownButton(index)}</Col>
            </Row>
          );
        })}
        <Button className={styles.addVariantBtn} onClick={onAddVariant} type={'link'}>
          {intl.formatMessage({ id: 'product.addVariants' })}
        </Button>
      </Col>
      <Col className={styles.rightHalfCol}>
        <Form
          onValuesChange={checkIfAllFillIn}
          form={form}
          className={styles.form}
          labelAlign={'left'}
          initialValues={dataToFormValues(editProduct.data.variants)}
        >
          {editProduct.data.variants?.map((e, index) => {
            return (
              <Row
                key={`${e.simpleSku}_${index}`}
                style={{ display: selectedVariant === index ? 'initial' : 'none' }}
              >
                <Col span={24}>
                  <Form.Item
                    key={`${e.simpleSku}_${index}`}
                    label={intl.formatMessage({ id: 'product.variantAttribute' })}
                    rules={[{ required: true }]}
                    name={`${index}_variantAttribute`}
                  >
                    <Select onChange={(value) => onVariantsSelectChange(index, value)}>
                      <Select.Option key={'package'}>Package</Select.Option>
                    </Select>
                  </Form.Item>
                </Col>
                {e.variantAttribute === 'package' ? (
                  <>
                    <Col span={24}>
                      <Form.Item
                        label={intl.formatMessage({ id: 'product.variantName' })}
                        rules={[{ required: false }]}
                        name={`${index}_variantName`}
                      >
                        <Input
                          key={'test'}
                          onBlur={(event) => onNameChange(index, event.target.value)}
                        />
                      </Form.Item>
                    </Col>
                    <Col span={24}>
                      <Form.Item
                        label={intl.formatMessage({ id: 'product.simpleSku' })}
                        rules={[{ required: true }]}
                        name={`${index}_simpleSku`}
                      >
                        <Input />
                      </Form.Item>
                    </Col>
                    <Col span={24}>
                      <Form.Item
                        label={intl.formatMessage({ id: 'product.weight' })}
                        rules={[
                          {
                            required: true,
                            validator: (_, value) =>
                              positiveIntegerValidate(_, value, intl, true),
                          },
                        ]}
                        name={`${index}_weight`}
                      >
                        <Input
                          type={'number'}
                          addonAfter={
                            <Select
                              onChange={(value) => onUomChange(index, value, 'weightUom')}
                              className={styles.addOnSelect}
                              getPopupContainer={(t) => t}
                              defaultValue={e.weightUom}
                            >
                              {dataCenter!.measureUnits.weight.map((weightUom) => {
                                return (
                                  <Select.Option key={weightUom.code}>
                                    {weightUom.symbol}
                                  </Select.Option>
                                );
                              })}
                            </Select>
                          }
                        />
                      </Form.Item>
                    </Col>
                    <Col>
                      <Row justify={'space-between'}>
                        <Col>
                          <Form.Item
                            labelCol={{ span: 24 }}
                            label={intl.formatMessage({
                              id: 'product.packageDimensions',
                            })}
                          />
                        </Col>
                        <Col className={styles.subRightHalf}>
                          <Form.Item
                            style={{ marginBottom: 0 }}
                            name={`${index}_height`}
                            label={intl.formatMessage({ id: 'product.height' })}
                            labelCol={{ span: 24 }}
                            rules={[
                              {
                                required: false,
                                validator: (_, value) =>
                                  positiveIntegerValidate(_, value, intl, false),
                              },
                            ]}
                          >
                            <Input
                              type={'number'}
                              addonAfter={
                                <Select
                                  onChange={(value) =>
                                    onUomChange(index, value, 'heightUom')
                                  }
                                  className={styles.addOnSelect}
                                  getPopupContainer={(t) => t}
                                  defaultValue={e.heightUom}
                                >
                                  {dataCenter!.measureUnits.length.map((unit) => {
                                    return (
                                      <Select.Option key={unit.code}>
                                        {unit.symbol}
                                      </Select.Option>
                                    );
                                  })}
                                </Select>
                              }
                            />
                          </Form.Item>
                          <Form.Item
                            style={{ marginBottom: 0 }}
                            name={`${index}_width`}
                            label={intl.formatMessage({ id: 'product.width' })}
                            labelCol={{ span: 24 }}
                            rules={[
                              {
                                required: false,
                                validator: (_, value) =>
                                  positiveIntegerValidate(_, value, intl, false),
                              },
                            ]}
                          >
                            <Input
                              type={'number'}
                              addonAfter={
                                <Select
                                  onChange={(value) =>
                                    onUomChange(index, value, 'widthUom')
                                  }
                                  className={styles.addOnSelect}
                                  getPopupContainer={(t) => t}
                                  defaultValue={e.widthUom}
                                >
                                  {dataCenter!.measureUnits.length.map((unit) => {
                                    return (
                                      <Select.Option key={unit.code}>
                                        {unit.symbol}
                                      </Select.Option>
                                    );
                                  })}
                                </Select>
                              }
                            />
                          </Form.Item>
                          <Form.Item
                            style={{ marginBottom: 0 }}
                            name={`${index}_length`}
                            label={intl.formatMessage({ id: 'product.length' })}
                            labelCol={{ span: 24 }}
                            rules={[
                              {
                                required: false,
                                validator: (_, value) =>
                                  positiveIntegerValidate(_, value, intl, false),
                              },
                            ]}
                          >
                            <Input
                              type={'number'}
                              addonAfter={
                                <Select
                                  className={styles.addOnSelect}
                                  getPopupContainer={(t) => t}
                                  defaultValue={e.lengthUom}
                                  onChange={(value) =>
                                    onUomChange(index, value, 'lengthUom')
                                  }
                                >
                                  {dataCenter!.measureUnits.length.map((lengthUom) => {
                                    return (
                                      <Select.Option key={lengthUom.code}>
                                        {lengthUom.symbol}
                                      </Select.Option>
                                    );
                                  })}
                                </Select>
                              }
                            />
                          </Form.Item>
                        </Col>
                      </Row>
                    </Col>
                  </>
                ) : null}
              </Row>
            );
          })}
        </Form>
      </Col>
      <Buttons
        step={editProduct.current}
        productId={editProduct.productId}
        isNewProduct={editProduct.isNew}
        disable={disable}
        onNextStep={onNext}
        onSave={async () => {
          const success = await onSubmit();
          if (success) setDisable({ save: true, nextStep: false });
          return success as boolean;
        }}
      />
    </Row>
  );
}
